@@ -5,7 +5,7 @@
 ;;;; Name:          db2-constants.lisp
 ;;;; Purpose:       Constants for CLSQL Db2 interface
 ;;;;
-;;;; $Id: db2-constants.lisp 9519 2004-05-30 06:22:22Z kevin $
+;;;; $Id: db2-constants.lisp 10893 2006-02-28 16:07:58Z kevin $
 ;;;;
 ;;;; This file is part of CLSQL.
 ;;;;
@@ -16,7 +16,7 @@
 
 (in-package #:clsql-db2)
 
-(defconstant SQL_NULL_HANDLE 0)
+(defconstant SQL_NULL_HANDLE nil)
 (defconstant SQL_HANDLE_ENV 1)
 (defconstant SQL_HANDLE_DBC 2)
 (defconstant SQL_HANDLE_STMT 3)
