@@ -7,7 +7,7 @@
 ;;;; Author:   Kevin M. Rosenberg
 ;;;; Created:  Feb 2002
 ;;;;
-;;;; $Id: mysql-loader.lisp 10115 2004-10-24 04:32:58Z kevin $
+;;;; $Id: mysql-loader.lisp 10612 2005-07-06 00:50:20Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002-2004 by Kevin M. Rosenberg
 ;;;;
@@ -18,24 +18,16 @@
 
 (in-package #:mysql)
 
-(defparameter *clsql-mysql-library-path* 
-  (uffi:find-foreign-library
-   '(#+64bit "mysql64" "mysql")
-   `(,clsql-mysql-system::*library-file-dir*
-     "/usr/lib/clsql/"
-     "/sw/lib/clsql/")
-   :drive-letters '("C")))
+(defparameter *clsql-mysql-library-candidate-names* 
+  (list #+(or 64bit x86-64) "clsql_mysql64"
+        #+(or 64bit x86-64) (make-pathname :name "clsql_mysql64"
+                                           :directory (pathname-directory *load-truename*))
+        "clsql_mysql"
+        (make-pathname :name "clsql_mysql"
+                       :directory (pathname-directory *load-truename*))))
 
 (defvar *mysql-library-candidate-names*
-    '("libmysqlclient" "libmysql"))
-
-(defparameter *mysql-library-candidate-directories*
-    `(,(pathname-directory *load-pathname*)
-      "/opt/mysql/lib/mysql/" "/usr/local/lib/"
-      #+64bit "/usr/lib64/" #+64bit "/usr/local/lib64/mysql/"
-      "/usr/lib/" "/usr/local/lib/mysql/" "/usr/lib/mysql/" "/mysql/lib/opt/" "/sw/lib/mysql/" "/opt/local/lib/mysql/"))
-
-(defvar *mysql-library-candidate-drive-letters* '("C" "D" "E"))
+  '("libmysqlclient" "libmysql"))
 
 (defvar *mysql-supporting-libraries* '("c")
   "Used only by CMU. List of library flags needed to be passed to ld to
@@ -49,21 +41,12 @@ set to the right path before compiling or loading the system.")
   *mysql-library-loaded*)
 				      
 (defmethod clsql-sys:database-type-load-foreign ((database-type (eql :mysql)))
-  (let ((mysql-path
-	 (uffi:find-foreign-library *mysql-library-candidate-names*
-				    *mysql-library-candidate-directories*
-				    :drive-letters
-				    *mysql-library-candidate-drive-letters*)))
-    (unless (probe-file mysql-path)
-      (error "Can't find mysql client library to load"))
-    (uffi:load-foreign-library mysql-path
-			       :module "mysql" 
-			       :supporting-libraries 
-			       *mysql-supporting-libraries*)
-    (uffi:load-foreign-library *clsql-mysql-library-path* 
-			       :module "clsql-mysql" 
-			       :supporting-libraries 
-			       (append *mysql-supporting-libraries*)))
+  (clsql-uffi:find-and-load-foreign-library *mysql-library-candidate-names*
+                                            :module "mysql" 
+                                            :supporting-libraries *mysql-supporting-libraries*)
+  (clsql-uffi:find-and-load-foreign-library *clsql-mysql-library-candidate-names*
+                                            :module "clsql-mysql" 
+                                            :supporting-libraries *mysql-supporting-libraries*)
   (setq *mysql-library-loaded* t))
 
 
