@@ -7,7 +7,7 @@
 ;;;; Authors:  Aurelio Bignoli, Kevin Rosenberg, Marcus Pearce
 ;;;; Created:  Aug 2003
 ;;;;
-;;;; $Id: sqlite-sql.lisp 10083 2004-10-03 15:45:47Z kevin $
+;;;; $Id: sqlite-sql.lisp 10275 2005-01-23 05:28:03Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2003 by Aurelio Bignoli and
 ;;;; Copyright (c) 2003-2004 by Kevin Rosenberg and Marcus Pearce.
@@ -174,8 +174,9 @@
       (loop for type in raw-types
 	    collect
 	    (case type
-	      ((:int :integer :tinyint :long)
+	      ((:int :integer :tinyint)
 	       :int32)
+	      (:long #+(or x86-64 64bit) :int64 #-(or x86-64 64bit) :int32)
 	      (:bigint
 	       :int64)
 	      ((:float :double)
