@@ -7,7 +7,7 @@
 ;;;; Authors:  Aurelio Bignoli
 ;;;; Created:  Oct 2004
 ;;;;
-;;;; $Id: sqlite3-sql.lisp 10191 2004-12-07 06:11:55Z kevin $
+;;;; $Id: sqlite3-sql.lisp 10275 2005-01-23 05:28:03Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2004 by Aurelio Bignoli
 ;;;;
@@ -103,7 +103,8 @@
 			(t :string))))
       (loop for type in result-types
 	    collect (case type
-		      ((:int :integer :tinyint :long) :int32)
+		      ((:int :integer :tinyint) :int32)
+		      (:long #+(or x86-64 64bit) :int64 #-(or x86-64 64bit) :int32)
 		      (:bigint :int64)
 		      ((:float :double) :double)
 		      ((:numeric) :number)
