@@ -9,7 +9,7 @@
 ;;;;                onShoreD to support UncommonSQL front-end 
 ;;;; Date Started:  Feb 2002
 ;;;;
-;;;; $Id: db-interface.lisp 10845 2005-11-26 16:08:12Z kevin $
+;;;; $Id: db-interface.lisp 10922 2006-05-03 14:39:10Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002-2004 by Kevin M. Rosenberg
 ;;;; and Copyright (c) 1999-2001 by Pierre R. Mai, and onShoreD
@@ -476,4 +476,4 @@ for foreign libraries \(in addition to the default places).")
 (defun push-library-path (path)
   "Adds the pathspec PATH \(which should denote a directory) to
 the list *FOREIGN-LIBRARY-SEARCH-PATHS*."
-  (push path *foreign-library-search-paths*))
\ No newline at end of file
+  (pushnew path *foreign-library-search-paths* :test #'equal))
