@@ -1,7 +1,7 @@
 ;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
 ;;;; *************************************************************************
 ;;;;
-;;;; $Id: generic-postgresql.lisp 10868 2006-01-16 21:46:13Z kevin $
+;;;; $Id: generic-postgresql.lisp 10969 2006-07-05 02:16:49Z kevin $
 ;;;;
 ;;;; Generic postgresql layer, used by db-postgresql and db-postgresql-socket
 ;;;;
@@ -77,7 +77,18 @@
   (mapcar #'car
 	  (database-query
 	   (format nil
-		   "SELECT relname FROM pg_class WHERE (relkind = '~A')~A"
+                   (if (not (eq owner :all))
+                    "
+ SELECT c.relname
+ FROM pg_catalog.pg_class c
+      LEFT JOIN pg_catalog.pg_roles r ON r.oid = c.relowner
+      LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
+ WHERE c.relkind IN ('~A','')
+       AND n.nspname NOT IN ('pg_catalog', 'pg_toast')
+       AND pg_catalog.pg_table_is_visible(c.oid)
+       ~A"
+                    "SELECT relname FROM pg_class WHERE (relkind =
+'~A')~A")
 		   type
 		   (owner-clause owner))
 	   database nil nil)))
