@@ -7,7 +7,7 @@
 ;;;; Programmers:   Kevin M. Rosenberg, Marc Battyani
 ;;;; Date Started:  Apr 2002
 ;;;;
-;;;; $Id: pool.lisp 10104 2004-10-23 03:56:54Z kevin $
+;;;; $Id: pool.lisp 10868 2006-01-16 21:46:13Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002-2003 by Kevin M. Rosenberg
 ;;;;
@@ -37,7 +37,8 @@
 	     (vector-pop (free-connections pool))))
       (let ((conn (connect (connection-spec pool)
 			   :database-type (pool-database-type pool)
-			   :if-exists :new)))
+			   :if-exists :new
+                           :make-default nil)))
 	(with-process-lock ((conn-pool-lock pool) "Acquire from pool")
 	  (vector-push-extend conn (all-connections pool))
 	  (setf (conn-pool conn) pool))
