@@ -3,7 +3,7 @@
 ;;;; File:     test-fddl.lisp
 ;;;; Authors:  Marcus Pearce <m.t.pearce@city.ac.uk> and Kevin Rosenberg
 ;;;; Created:  30/03/2004
-;;;; Updated:  $Id: test-fddl.lisp 9831 2004-08-03 06:27:00Z kevin $
+;;;; Updated:  $Id: test-fddl.lisp 10845 2005-11-26 16:08:12Z kevin $
 ;;;;
 ;;;; Tests for the CLSQL Functional Data Definition Language
 ;;;; (FDDL).
@@ -88,9 +88,13 @@
                           '(:postgresql :postgresql-socket))
                   :ignore
                   :warn)))
-         (clsql:create-table [foo] 
-                             '(([bar] integer :not-null :unique :primary-key) 
-                               ([baz] string :not-null :unique))))
+         (case *test-database-underlying-type*
+           (:mssql (clsql:create-table [foo] 
+                                       '(([bar] integer :not-null :primary-key)
+                                         ([baz] string :not-null :unique))))
+           (t (clsql:create-table [foo] 
+                                  '(([bar] integer :not-null :unique :primary-key)
+                                    ([baz] string :not-null :unique))))))
        (clsql:table-exists-p [foo]))
      (progn
        (clsql:drop-table [foo])
