@@ -3,7 +3,7 @@
 ;;;; File:    test-init.lisp
 ;;;; Authors: Marcus Pearce <m.t.pearce@city.ac.uk>, Kevin Rosenberg
 ;;;; Created: 30/03/2004
-;;;; Updated: $Id: test-init.lisp 10845 2005-11-26 16:08:12Z kevin $
+;;;; Updated: $Id: test-init.lisp 10922 2006-05-03 14:39:10Z kevin $
 ;;;;
 ;;;; Initialisation utilities for running regression tests on CLSQL. 
 ;;;;
@@ -623,11 +623,14 @@
           ((and (eq *test-database-underlying-type* :mssql)
                 (clsql-sys:in test :fdml/select/9))
            (push (cons test "mssql uses integer math for AVG") skip-tests))
+          ((and (not (member *test-database-underlying-type* 
+                             '(:postgresql :mysql :sqlite3)))
+                (clsql-sys:in test :fdml/select/37 :fdml/select/38))
+           (push (cons test "LIMIT keyword not supported in SELECT") skip-tests))
 	  (t
 	   (push test-form test-forms)))))
       (values (nreverse test-forms) (nreverse skip-tests))))
 
-
 (defun rapid-load (type &optional (position 0))
   "Rapid load for interactive testing."
   (when *default-database*
