@@ -3,7 +3,7 @@
 ;;;; File:    test-oodml.lisp
 ;;;; Author:  Marcus Pearce <m.t.pearce@city.ac.uk>
 ;;;; Created: 01/04/2004
-;;;; Updated: $Id: test-oodml.lisp 9812 2004-07-23 21:19:49Z marcusp $
+;;;; Updated: $Id: test-oodml.lisp 10331 2005-02-17 23:47:29Z kevin $
 ;;;;
 ;;;; Tests for the CLSQL Object Oriented Data Definition Language
 ;;;; (OODML).
@@ -50,11 +50,11 @@
 				     " "
 				     (slot-value e 'last-name)))
 	     (clsql:select 'employee :where [= [slot-value 'employee 'first-name]
-					       "Vladamir"]
+					       "Vladimir"]
 			   :flatp t		     
 			   :order-by [last-name]
 			   :caching nil))
-	  ("Vladamir Lenin" "Vladamir Putin"))
+	  ("Vladimir Lenin" "Vladimir Putin"))
 
 	(deftest :oodml/select/5
 	    (length (clsql:select 'employee :where [married] :flatp t :caching nil))
@@ -194,7 +194,7 @@
 			      ": "
 			      (employee-email lenin))))
 	     (progn 
-	       (setf (slot-value employee1 'first-name) "Vladamir" 
+	       (setf (slot-value employee1 'first-name) "Vladimir" 
 		     (slot-value employee1 'last-name) "Lenin"
 		     (slot-value employee1 'email) "lenin@soviet.org")
 	       (clsql:update-records-from-instance employee1)
@@ -209,9 +209,9 @@
 			      (last-name lenin)
 			      ": "
 			      (employee-email lenin)))))
-	  "Vladamir Lenin: lenin@soviet.org"
+	  "Vladimir Lenin: lenin@soviet.org"
 	  "Dimitriy Ivanovich: ivanovich@soviet.org"
-	  "Vladamir Lenin: lenin@soviet.org")
+	  "Vladimir Lenin: lenin@soviet.org")
 
 	;; tests update-record-from-slot 
 	(deftest :oodml/update-records/2
@@ -270,7 +270,7 @@
 			      ": "
 			      (employee-email lenin))))
 	     (progn 
-	       (setf (slot-value employee1 'first-name) "Vladamir" 
+	       (setf (slot-value employee1 'first-name) "Vladimir" 
 		     (slot-value employee1 'last-name) "Lenin"
 		     (slot-value employee1 'email) "lenin@soviet.org")
 	       (clsql:update-record-from-slots employee1 '(first-name last-name email))
@@ -285,9 +285,9 @@
 			      (last-name lenin)
 			      ": "
 			      (employee-email lenin)))))
-	  "Vladamir Lenin: lenin@soviet.org"
+	  "Vladimir Lenin: lenin@soviet.org"
 	  "Dimitriy Ivanovich: ivanovich@soviet.org"
-	  "Vladamir Lenin: lenin@soviet.org")
+	  "Vladimir Lenin: lenin@soviet.org")
 
 	;; tests update-instance-from-records 
 	(deftest :oodml/update-instance/1
@@ -313,7 +313,7 @@
 			    (slot-value employee1 'email)))
 	     (progn 
 	       (clsql:update-records [employee] 
-				     :av-pairs '(([first-name] "Vladamir")
+				     :av-pairs '(([first-name] "Vladimir")
 						 ([last-name] "Lenin")
 						 ([email] "lenin@soviet.org"))
 				     :where [= [emplid] 1])
@@ -324,9 +324,9 @@
 			    (slot-value employee1 'last-name)
 			    ": "
 			    (slot-value employee1 'email))))
-	  "Vladamir Lenin: lenin@soviet.org"
+	  "Vladimir Lenin: lenin@soviet.org"
 	  "Ivan Petrov: petrov@soviet.org"
-	  "Vladamir Lenin: lenin@soviet.org")
+	  "Vladimir Lenin: lenin@soviet.org")
 
 	;; tests update-slot-from-record 
 	(deftest :oodml/update-instance/2
@@ -454,6 +454,16 @@
 		 (every #'(lambda (dea) (typep (slot-value dea 'address) 'address)) dea-list)
 		 (mapcar #'(lambda (dea) (slot-value (slot-value dea 'address) 'addressid)) dea-list))))
 	  t t t t (1 1 2 2 2))
+
+	;; update-object-joins needs to be fixed for multiple keys
+	#+ignore
+	(deftest :oodml/uoj/2
+	    (progn
+	      (clsql:update-objects-joins (list company1))
+	      (mapcar #'(lambda (e)
+			  (slot-value e 'ecompanyid))
+	         (company-employees company1)))
+	  (1 1 1 1 1 1 1 1 1 1))
 	
 	(deftest :oodml/big/1 
 	    (let ((objs (clsql:select 'big :order-by [i] :flatp t)))
