@@ -7,7 +7,7 @@
 ;;;; Programmers:   Kevin M. Rosenberg
 ;;;; Date Started:  Mar 2002
 ;;;;
-;;;; $Id: clsql-uffi.lisp 10108 2004-10-23 15:16:15Z kevin $
+;;;; $Id: clsql-uffi.lisp 10238 2004-12-29 07:43:08Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002 by Kevin M. Rosenberg
 ;;;;
@@ -57,6 +57,8 @@
 	      t))
 	   (:blob
 	    :blob)
+	   (:uint
+	    :uint)
 	   (t
 	    t))
 	 new-types))))
@@ -65,6 +67,12 @@
     ((str (* :unsigned-char)))
   :returning :int)
 
+(uffi:def-function ("strtoul" c-strtoul)
+    ((str (* :unsigned-char))
+     (endptr (* :unsigned-char))
+     (radix :int))
+  :returning :unsigned-long)
+
 (uffi:def-function "atol"
     ((str (* :unsigned-char)))
   :returning :long)
@@ -97,6 +105,11 @@
 
 (uffi:def-type char-ptr-def (* :unsigned-char))
 
+(defun strtoul (char-ptr)
+  (declare (optimize (speed 3) (safety 0) (space 0))
+ 	   (type char-ptr-def char-ptr))
+  (c-strtoul char-ptr uffi:+null-cstring-pointer+ 10))
+
 (defun convert-raw-field (char-ptr types index &optional length)
   (declare (optimize (speed 3) (safety 0) (space 0))
  	   (type char-ptr-def char-ptr))
@@ -110,9 +123,15 @@
        (case type
 	 (:double
 	  (atof char-ptr))
-	 ((:int32 :int)
+	 (:int
+	  (atol char-ptr))
+	 (:int32
 	  (atoi char-ptr))
-	 (:int64
+	 (:uint32
+	  (strtoul char-ptr))
+	 (:uint
+	  (strtoul char-ptr))
+	 ((:int64 :uint64)
 	  (uffi:with-foreign-object (high32-ptr :unsigned-int)
 	    (let ((low32 (atol64 char-ptr high32-ptr))
 		  (high32 (uffi:deref-pointer high32-ptr :unsigned-int)))
@@ -124,6 +143,11 @@
 	      (uffi:convert-from-foreign-usb8 char-ptr length)
 	    (error "Can't return blob since length is not specified.")))
 	 (t
+	  ;; sb-unicode still broken with converting with length, assume
+	  ;; that string is null terminated
+	  #+sb-unicode
+	  (uffi:convert-from-foreign-string char-ptr :locale :none)
+	  #-sb-unicode
           (if length
 	      (uffi:convert-from-foreign-string char-ptr :locale :none
                                                 :null-terminated-p nil
